local fzflua_status_ok, fzflua = pcall(require, 'fzf-lua')
if not fzflua_status_ok then
  vim.notify("fzf-lua plugin not found!")
  return
end

local actions = require "fzf-lua.actions"

fzflua.setup
{
  global_resume = true,
  global_resume_query = true,

  winopts =
  {
    height = 0.60,
    width  = 0.70,
    fullscreen = true,
    preview =
    {
      horizontal     = 'right:50%',
      vertical       = 'down:55%',
      layout         = 'horizontal',
    }
  },

  -------------------------
  keymap =
  {
    builtin =
    {
      ["<C-f>"] = "preview-page-down",
      ["<C-b>"] = "preview-page-up",
      ["<C-r>"] = "preview-page-reset",
      ["<F1>"]  = "toggle-help",
      ["<F2>"]  = "toggle-fullscreen",
      ["<F3>"]  = "toggle-preview-wrap",
      ["<F4>"]  = "toggle-preview",
      ["<F5>"]  = "toggle-preview-ccw",
      ["<F6>"]  = "toggle-preview-cw",
    },
    fzf =
    {
      ["ctrl-a"] = "beginning-of-line",
      ["ctrl-e"] = "end-of-line",
      ["ctrl-d"] = "half-page-down",
      ["ctrl-u"] = "half-page-up",
      ["ctrl-n"] = "down",
      ["ctrl-p"] = "up",
      ["alt-n"]  = "next-history",
      ["alt-p"]  = "previous-history",
      ["alt-a"]  = "toggle-all",
    }
  },

  -------------------------
  actions =
  {
    files =
    {
      ["default"]     = actions.file_edit,
      ["ctrl-s"]      = actions.file_split,
      ["ctrl-v"]      = actions.file_vsplit,
      ["ctrl-t"]      = actions.file_tabedit,
      ["alt-q"]       = actions.file_sel_to_qf,
      ["alt-l"]       = actions.file_sel_to_ll,
    },
    buffers =
    {
      ["default"]     = actions.buf_edit,
      ["ctrl-s"]      = actions.buf_split,
      ["ctrl-v"]      = actions.buf_vsplit,
      ["ctrl-t"]      = actions.buf_tabedit,
    }
  },

  -------------------------
  files =
  {
    cmd = [[ fdfind --type file --no-ignore --hidden --follow \
                    --exclude "**/obj-x86_64-linux-gnu/**" \
                    --exclude "**/.git/**" \
                    --exclude "**/.cache/**" ]],
    fzf_opts =
    {
      ['--history'] = vim.fn.stdpath("data") .. '/fzf-lua-files-history'
    },
    winopts =
    {
      preview =
      {
        horizontal     = 'right:50%',
        layout         = 'horizontal',
      }
    }
  },

  -------------------------
  grep =
  {
    debug=false,
    no_esc=true,  -- don't escape \
    cmd = [[ rg --with-filename --no-heading --line-number --column --smart-case \
                --no-ignore --hidden --trim --color "always" --sort-files \
                --glob "!{**/.git/**,**/obj-x86_64-linux-gnu/**,**/.cache/**,**/*.so,**/debian/**/*}" -e ]],
    fzf_opts =
    {
      ['--history'] = vim.fn.stdpath("data") .. '/fzf-lua-grep-history',
      ['--tiebreak'] = "index"
    },
  },

  -------------------------
  lsp =
  {
    code_actions =
    {
      winopts =
      {
        fullscreen = false,
        height = 0.20,
        width  = 0.40,
        row    = 0.50,
        col    = 0.50,
      }
    }
  }
}


----------------------------------------------------------------
-- keymapping
----------------------------------------------------------------
-- keymapping functions
local km = require('user/modules/keymapping')

-- Misc search
km.nnoremap('<leader>fv', ':FzfLua grep_visual<CR>')
km.nnoremap('<leader>fs', ':FzfLua resume<CR>')

-- Buffers, files and lines
km.nnoremap('<leader>b' , ':FzfLua buffers<CR>')
km.nnoremap('<leader>ff', ':FzfLua files<CR>')
km.nnoremap('<leader>fl', ':FzfLua blines<CR>')
km.nnoremap('<leader>fL', ':FzfLua lines<CR>')

-- Current word/WORD in buffers and lines
km.nnoremap('<leader>fj', ':FzfLua blines search=<C-r><C-w><CR>')
km.nnoremap('<leader>fJ', ':FzfLua blines search=<C-r><C-a><CR>')

-- Current word/WORD in files, automatically or with edit
-- Note: use <C-l> to redraw the screen and make sure the query is
-- visible if there is no <CR> at the end.
km.nnoremap('<leader>fd', ':FzfLua grep search=<C-l>')
km.nnoremap('<leader>fe', ':FzfLua grep search=<C-r><C-w><C-l>')
km.nnoremap('<leader>fE', ':FzfLua grep search=<C-r><C-a><C-l>')
-- CTRL+W and CTRL-A don't work well when using \b and searching for
-- a pattern that starts with 'b', due to the fact that the 'b' in
-- "\b" is considered as part of the work under cursor. This results
-- in a search string which is missing a 'b' character. To avoid this,
-- it is necessary to first insert the work under cursor, then go back
-- and add "\b" in front of it.
km.nnoremap('<leader>fw', ':FzfLua grep search=<C-r><C-w>\\b<C-Left><Right><Right><Right><Right><Right><Right><Right>\\b<CR>')
km.nnoremap('<leader>fW', ':FzfLua grep search=<C-r><C-a>\\b<C-Left><Right><Right><Right><Right><Right><Right><Right>\\b<CR>')

-- LSP
km.nnoremap('<leader>fr', ':FzfLua lsp_references<CR>')
km.nnoremap('<leader>ft', ':FzfLua lsp_typedefs<CR>')
km.nnoremap('<leader>fi', ':FzfLua lsp_incoming_calls<CR>')
km.nnoremap('<leader>fy', ':FzfLua lsp_workspace_symbols<CR>')
km.nnoremap('<leader>fo', ':FzfLua lsp_code_actions<CR>')
km.nnoremap('<leader>fa', ':FzfLua lsp_document_diagnostics<CR>')
km.nnoremap('<leader>fG', ':FzfLua lsp_workspace_diagnostics<CR>')

-- History, Marks, Maps
km.nnoremap('<leader>fh', ':FzfLua oldfiles<CR>')
km.nnoremap('<leader>f:', ':FzfLua command_history<CR>')
km.nnoremap('<leader>f/', ':FzfLua search_history<CR>')
km.nnoremap('<leader>fn', ':FzfLua marks<CR>')
km.nnoremap('<leader>fp', ':FzfLua keymaps<CR>')

-- Git
km.nnoremap('<leader>fc', ':FzfLua git_commits<CR>')
km.nnoremap('<leader>fu', ':FzfLua git_status<CR>')

km.nnoremap('<leader>fm', ':FZFMru<CR>')

-- Alternative way to specify options in direct calls. This is useful when
-- options in setup {} are not available/implemented.
--vim.keymap.set('n', '<leader>fc', function() return fzflua.lsp_code_actions({winopts = { fullscreen = false, height = 0.20, width = 0.40, row = 0.40 }}) end, { noremap = true, silent = true})

