local cmp_status_ok, nvimtree = pcall(require, 'nvim-tree')
if not cmp_status_ok then
  vim.notify("nvim-tree plugin not found!")
  return
end

local api = require "nvim-tree.api"

local function edit_and_close()
  local node = api.tree.get_node_under_cursor()
  if node.nodes ~= nil then
    -- expand or collapse folder
    api.node.open.edit()
  else
    -- open file with picker window and close tree
    api.node.open.edit()
    api.tree.close()
  end
end


local function edit_no_picker_and_close()
  local node = api.tree.get_node_under_cursor()
  if node.nodes ~= nil then
    -- expand or collapse folder
    api.node.open.edit()
  else
    -- open file without picker window and close tree
    api.node.open.no_window_picker()
    api.tree.close()
  end
end


local function select_folder()
  local node = api.tree.get_node_under_cursor()
  if node.nodes ~= nil then
    -- expand or collapse folder and disable filter if active
    api.tree.change_root_to_node()
		api.live_filter.clear()
  end
end


local function open_in_tab_silent(node)
  api.node.open.tab(node)
  vim.cmd.tabprev()
end


local function my_on_attach(bufnr)

  local function opts(desc)
    return { desc = "nvim-tree: " .. desc, buffer = bufnr, noremap = true, silent = true, nowait = true }
  end

  -- default mappings
  api.config.mappings.default_on_attach(bufnr)

	-- remove default mappings
  vim.keymap.del('n', '-', { buffer = bufnr })
  vim.keymap.del('n', '<BS>', { buffer = bufnr })
  vim.keymap.del('n', '<C-e>', { buffer = bufnr })
  vim.keymap.del('n', '<C-r>', { buffer = bufnr })
  vim.keymap.del('n', '<C-x>', { buffer = bufnr })
  vim.keymap.del('n', '<2-LeftMouse>', { buffer = bufnr })
  vim.keymap.del('n', '<2-RightMouse>', { buffer = bufnr })
  vim.keymap.del('n', '[e', { buffer = bufnr })
  vim.keymap.del('n', ']e', { buffer = bufnr })
	vim.keymap.del('n', 'g?', { buffer = bufnr })
  vim.keymap.del('n', 'bmv', { buffer = bufnr })
  vim.keymap.del('n', 'B', { buffer = bufnr })
  vim.keymap.del('n', 'I', { buffer = bufnr })
  vim.keymap.del('n', 'J', { buffer = bufnr })
  vim.keymap.del('n', 'K', { buffer = bufnr })
  vim.keymap.del('n', 'o', { buffer = bufnr })
  vim.keymap.del('n', 'O', { buffer = bufnr })
  vim.keymap.del('n', 'P', { buffer = bufnr })
	vim.keymap.del('n', 'R', { buffer = bufnr })
  vim.keymap.del('n', 's', { buffer = bufnr })
  vim.keymap.del('n', 'S', { buffer = bufnr })
  vim.keymap.del('n', 'U', { buffer = bufnr })
  vim.keymap.del('n', '>', { buffer = bufnr })
  vim.keymap.del('n', '<', { buffer = bufnr })

	-- set new custom mappings
  vim.keymap.set('n', '<C-s>', api.node.open.horizontal,           opts('Open: Horizontal Split'))

	vim.keymap.set('n', 'I',     api.node.navigate.sibling.first,    opts('First Sibling'))
  vim.keymap.set('n', 'J',     api.node.navigate.sibling.next,     opts('Next Sibling'))
  vim.keymap.set('n', 'K',     api.node.navigate.sibling.prev,     opts('Previous Sibling'))
	vim.keymap.set('n', 'U',     api.node.navigate.sibling.last,     opts('Last Sibling'))

	vim.keymap.set('n', 'h',     api.node.navigate.parent,           opts('Parent Directory'))
  vim.keymap.set('n', 'L',     api.node.open.edit,                 opts('Open'))
  vim.keymap.set('n', 'l',     api.node.open.no_window_picker,     opts('Open: No Window Picker'))
  vim.keymap.set('n', 'O',     edit_and_close,                     opts('Edit + Close Tree'))
  vim.keymap.set('n', 'o',     edit_no_picker_and_close,           opts('Edit (No Picker) + Close Tree'))
  vim.keymap.set('n', 's',     select_folder,                      opts('Select Folder'))
  vim.keymap.set('n', 'w',     api.tree.change_root_to_parent,     opts('Select Parent'))
	vim.keymap.set('n', 'T',     open_in_tab_silent,                 opts('Open in Silent Tab'))

  vim.keymap.set('n', '<C-r>', api.tree.reload,                    opts('Refresh'))
  vim.keymap.set('n', 'R',     api.fs.rename_sub,                  opts('Rename: Omit Filename'))
  vim.keymap.set('n', 'X',     api.node.run.system,                opts('Exec System Command'))

  vim.keymap.set('n', ']b',    api.marks.navigate.next,            opts('Next bookmark'))
  vim.keymap.set('n', '[b',    api.marks.navigate.prev,            opts('Prev bookmark'))

  vim.keymap.set('n', ']w',    api.node.navigate.diagnostics.next, opts('Next Diagnostic'))
  vim.keymap.set('n', '[w',    api.node.navigate.diagnostics.prev, opts('Prev Diagnostic'))

  vim.keymap.set('n', '?',     api.tree.toggle_help,               opts('Help'))
end


nvimtree.setup(
{
	on_attach = my_on_attach,
	disable_netrw = true,
	hijack_netrw = true,
	view = {
		width = {
			min = 35
		},
	},
	renderer = {
		indent_markers = {
			enable = true,
		},
		icons = {
			webdev_colors = true,
			git_placement = "after",
			modified_placement = "after",
		},
	},
	update_focused_file = {
		update_root = true,
	},
	diagnostics = {
		enable = true,
		show_on_dirs = true,
		show_on_open_dirs = true,
		debounce_delay = 50,
		severity = {
			min = vim.diagnostic.severity.HINT,
			max = vim.diagnostic.severity.ERROR,
		},
		icons = {
			--hint = "",
			--info = "",
			--warning = "",
			--error = "",
			hint = "H",
			info = "I",
			warning = "W",
			error = "E",
		},
	},
	filters = {
		dotfiles = false,
		git_clean = false,
		no_buffer = false,
		custom = {},
		exclude = {},
	},
	live_filter = {
		prefix = "Filter:",
		always_show_folders = false,
	},
	actions = {
		change_dir = {
			global = true,
		},
	},
})


----------------------------------------------------------------
-- keymapping
----------------------------------------------------------------
-- keymapping functions
local km = require('user/modules/keymapping')

-- Buffers, files and lines
km.nnoremap('<leader>gn' , ':NvimTreeToggle<CR>')
km.nnoremap('<leader>gf' , ':NvimTreeFindFile<CR>')
